==> ev_framework-0.0.4.tar.gz: lib/Elves/Chatroom/_Ev_Chat_View.dart <==
import 'dart:io';

import '../API/Ev_API.dart';
import '../Image/Ev_Image_Widget.dart';
import '../Image/Ev_PhotoView.dart';
import '../System/Ev_Firebase.dart';
import '../System/Ev_Func.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import '_Ev_Send_File.dart';
import '_Ev_Send_Message.dart';
import '_Ev_Send_Photo.dart';
import '_Ev_Send_query.dart';


class $Ev_Chat_View extends StatefulWidget {
   var url;
   Map<String,String> post;
   Color Self_bgcolor;
   Color Other_bgcolor;
   Color self_fontcolor;
   Color other_fontcolor;

   String Send_message_url;
   int last_id = 0;


   $Ev_Chat_View({this.url,this.post,this.Self_bgcolor,this.Other_bgcolor,this.self_fontcolor,this.other_fontcolor,this.Send_message_url,Key key}): super(key: key)
   {}

   @override
  $Ev_Chat_ViewtState createState() => $Ev_Chat_ViewtState();
}

class $Ev_Chat_ViewtState extends State<$Ev_Chat_View> implements Ev_FireBase_CallBack {

  Ev_API api = Ev_API();
  Send_Query send_query;
  var loaded = false;
  var error = false;

  List data;

  ScrollController _controller;

  void initState() {
    _controller = ScrollController();
    _controller.addListener(_scrollListener);
    send_query = Send_Query(chat_view: widget);
    Ev_Firebase.initState_CallBack(this);
  }

  _scrollListener() {
    if (_controller.offset >= _controller.position.maxScrollExtent &&
        !_controller.position.outOfRange) {
    }
    if (_controller.offset <= _controller.position.minScrollExtent &&
        !_controller.position.outOfRange) {
    }
  }

  @override
  void dispose() {
    api.Stop();
    send_query.dispose();
    Ev_Firebase.dispose(this);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if(error)
    {  return _Error_Reload(context); }


    if(!loaded) {
      print(widget.post);
      Map<String,String> post = widget.post;
      post["last_id"] = "0";

      api.Post( widget.url, post: post, alert_error: true,
          onSuccess: (Ev_API_Result result) {
            widget.last_id = int.parse(result.sdata["last_id"]);
            setState(() {
              data = result.data;
              loaded = true;
              error = false;
            });
          },
          onConnectError: () {
            print("On Connect Error");
            setState(() {
              error = true;
              loaded = false;
            });
          },
          onReturnError:()
          {  print("On Return Error !");
            setState((){
            error = true;
            loaded = false;
          });
          }
      );
      return Center(
          child: SizedBox(
              child:
              new CircularProgressIndicator(
                  valueColor: new AlwaysStoppedAnimation(Colors.blue),
                  strokeWidth: 5.0),
              height: 40.0,
              width: 40.0));
    }
   else
    {   return _listview(); }
  }


  Widget _Error_Reload(BuildContext context)
  {  return Center(child: Column(
    mainAxisAlignment: MainAxisAlignment.center,
    children: <Widget>[
      Icon(Icons.error_outline,color: Colors.amber,size: 110),
      RaisedButton(child: Icon(Icons.refresh,size: 25,),onPressed: (){
        setState(() {
          error = false;
          loaded = false;
        });
      },)
    ],),);
  }


  Widget _listview()
  { print("List Build item " + data.length.toString());
    return GestureDetector(
      child: ListView.builder(itemCount: data.length,
        padding: EdgeInsets.all(10.0),
        controller: _controller,
        reverse: true,
        itemBuilder: (context, index) {
        return Container(width: double.infinity,
            alignment: (data[index]["type"] == "1") ? Alignment.topRight : Alignment.topLeft,
            child: message_item(data[index])
        );
     }),
    onTap: (){ print("Tap "); FocusScope.of(context).requestFocus(FocusNode());

    Load_Message();
    },
    onTapDown: (TapDownDetails details)  { print("Tap Down"); FocusScope.of(context).requestFocus(FocusNode());},
   );
  }

  Widget message_item(sdata)
  {   if(sdata["type"] == "1" || sdata["member_icon"]=="")
        return message_item_body(sdata);
      else
        return Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
           Padding(
             padding: const EdgeInsets.only(top:5.0),
             child: ClipRRect(
                 borderRadius: BorderRadius.circular(15),
                 child: Image.network(sdata["member_icon"],width: 30,height: 30,fit: BoxFit.cover,),),
           ),
            message_item_body(sdata)
        ],);


  }

  Widget message_item_body(sdata) {

    Color fontcolor = (sdata["type"] == "1") ? widget.self_fontcolor : widget.other_fontcolor;
    Color bg_color = (sdata["type"] == "1") ? widget.Self_bgcolor : widget.Other_bgcolor;

    if (sdata["send_out"] == "1" ) {
       return sdata["widget"];
    }

    if (sdata["message_type"] == "0") {
      return Card(
        child: ClipRRect(
            borderRadius: BorderRadius.circular(5.0),
            child: Stack(
              children: <Widget>[
                Container(
                  padding: EdgeInsets.symmetric(horizontal: 10,vertical: 5),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[

                      (sdata["type"]) == "0" && sdata["member_name"]!="" ? Text(
                          sdata["member_name"],
                          style: TextStyle(color: Color(
                              int.parse(sdata["member_color"], radix: 16)))): Container(width:0),

                      Text(sdata["message"].toString(),
                        style: TextStyle(color: fontcolor),),


                      SizedBox(height: 13,),
                    ],
                  ),
                  color: bg_color,
                  constraints: BoxConstraints(
                    maxWidth: 300,
                    minWidth: 100,
                  ),
                ),

                Positioned(
                  right: 5,
                  bottom: 3,
                  child: Text(sdata["time"], style: TextStyle(fontSize: 10,
                      color: fontcolor.withOpacity(0.7)),
                    textAlign: TextAlign.right,),
                ),
              ],
            )),
      );
    }

    if (sdata["message_type"] == "1") {
      return Card(
        child: ClipRRect(
            borderRadius: BorderRadius.circular(5.0),
            child: Container(
              padding: EdgeInsets.all(4.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  (sdata["type"]) == "0" && sdata["member_name"]!="" ? Text(
                      sdata["member_name"],
                      style: TextStyle(color: Color(
                          int.parse(sdata["member_color"], radix: 16)))): Container(width:0),

                  Stack(children: <Widget>[
                    SizedBox(width: 200, height: 200,
                        child: GestureDetector(
                          child: Ev_Image_Widget(image: sdata["file"],
                            boxfit: BoxFit.cover,
                            round: 5,),
                          onTap: () {
                            Navigator.push(context, MaterialPageRoute(builder: (
                                context) => Ev_PhotoView(image: sdata["file"])));
                          },
                        )),
                    Positioned(right:5,bottom:0, child:
                    Padding(
                      padding: const EdgeInsets.all(5.0),
                      child: Text(sdata["time"],
                          style: TextStyle(fontSize: 10, color: Colors.white)),
                    ),)
                  ],),
                ],
              ),
              color: sdata["type"] == "1" ? widget.Self_bgcolor : widget.Other_bgcolor,
            )),
      );
    }

    if (sdata["message_type"] == "2") {
      return GestureDetector(
        onTap: (){ Ev_Func.Open_URL(sdata["file"]); },
        child: Stack(
          children: <Widget>[
            Card(
              child: ClipRRect(
                  borderRadius: BorderRadius.circular(5.0),
                  child: Container(
                      color: bg_color,
                      constraints: BoxConstraints(
                        maxWidth: 220,
                        minWidth: 100,),

                      padding: EdgeInsets.all(6.0),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                        (sdata["type"]) == "0" && sdata["member_name"]!="" ? Text(
                            sdata["member_name"],
                            style: TextStyle(color: Color(
                                int.parse(sdata["member_color"], radix: 16)))): Container(width:0),

                        SizedBox(height: 2,),
                        Row(children: <Widget>[
                          ClipRRect(borderRadius: BorderRadius.circular(15.0),
                              child: Container(width: 30, height: 30,
                                color: Colors.blue,
                                child: Center(child: Icon(
                                    Icons.insert_drive_file, size: 20,
                                    color: Colors.white)),)
                          ),

                          SizedBox(width: 8,),
                          Expanded(
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                              Text(sdata["filename"].toString(), overflow: TextOverflow.ellipsis,
                                style: TextStyle(color: fontcolor),),
                              Text(sdata["filesize"], style: TextStyle(fontSize: 10,
                                  color: fontcolor.withOpacity(0.7)),),
                            ],),
                          ),
                        ],),
                      ],))),
            ),
            Positioned(
              right: 15,
              bottom: 8,
              child: Text(sdata["time"], style: TextStyle(fontSize: 10,
                  color: fontcolor.withOpacity(0.7)),
                textAlign: TextAlign.right,),
            ),
          ],
        ),
         );
      }
  }


  void Send_Message(String message)
  {   if(message.replaceAll(" ","") == "" )
         { return; }
      Map<String,dynamic> sdata = Map();
      sdata["message"] = message;
      sdata["type"] = "1";
      sdata["send_out"] = "1";
      sdata["message_type"] = "0";
      sdata["time"] = DateTime.now().toString().substring(11,16);
      sdata["widget"] =  $EV_Send_Message(key:GlobalKey(),url: widget.Send_message_url,
      fontcolor: widget.self_fontcolor,
      bgcolor: widget.Self_bgcolor,
      post: widget.post,
      sdata: sdata,
      send_query: send_query,
      onSuccess: Send_Out_Received);

      setState(() {
        data.insert(0,sdata);
      });

  }

  void Send_Photo(File file)
  {    Map<String,dynamic> sdata = Map();
       sdata["message"] = "";
       sdata["type"] = "1";
       sdata["send_out"] = "1";
       sdata["photo"] = file;
       sdata["message_type"] = "1";
       sdata["time"] = DateTime.now().toString().substring(11,16);

       sdata["widget"] =  $EV_Send_Photo(key:GlobalKey(),url: widget.Send_message_url,
        fontcolor: widget.self_fontcolor,
        bgcolor: widget.Self_bgcolor,
        post: widget.post,
        sdata: sdata,
        send_query: send_query,
        onSuccess: Send_Out_Received);

     setState(() {
        data.insert(0,sdata);
     });

  }

  void Send_File(File file)
  {    Map<String,dynamic> sdata = Map();
       sdata["message"] = "";
       sdata["type"] = "1";
       sdata["send_out"] = "1";
       sdata["file"] = file;
       sdata["message_type"] = "1";
       sdata["time"] = DateTime.now().toString().substring(11,16);

      sdata["widget"] =  $EV_Send_File(key:GlobalKey(),url: widget.Send_message_url,
      fontcolor: widget.self_fontcolor,
      bgcolor: widget.Self_bgcolor,
      post: widget.post,
      sdata: sdata,
      send_query: send_query,
      onSuccess: Send_Out_Received);

  setState(() {
    data.insert(0,sdata);
  });


  }

  void Send_Out_Received(Ev_API_Result result)
   { print("Rescive " + result.toString()) ;
     var id = int.parse(result.sdata["last_id"]);
     if(id>widget.last_id)
         { widget.last_id = id; }

     setState(() {
       result.data.forEach((dynamic value) {
         data.insert(0,value);
       }); });
   }

  void Load_Message()
  { if(send_query.action == true)
       { return; }

    var post = widget.post;
         post["last_id"] = widget.last_id.toString();

     api.Post(widget.url, post: post,
      onSuccess: (Ev_API_Result result) {
       widget.last_id =  int.parse(result.sdata["last_id"]);
       setState(() {
          result.data.forEach((dynamic value) {
            data.insert(0,value);

          });

        if(result.data.length >0)
           {  //_controller.animateTo(0.0, duration: Duration(milliseconds: 300), curve: Curves.easeOut);

              SystemSound.play(SystemSoundType.click);
           }

        });
      });
   }

  @override
  FireBase_OnMessage(message) {
     Load_Message();
  }




}

==> ev_framework-0.0.8.tar.gz: lib/Elves/Chatroom/_Ev_Chat_View.dart <==
import 'dart:io';

import 'package:ev_framework/Elves/System/Ev_Timer.dart';

import '../API/Ev_API.dart';
import '../Image/Ev_Image_Widget.dart';
import '../Image/Ev_PhotoView.dart';
import '../System/Ev_Firebase.dart';
import '../System/Ev_Func.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import '_Ev_Send_File.dart';
import '_Ev_Send_Message.dart';
import '_Ev_Send_Photo.dart';
import '_Ev_Send_query.dart';


class $Ev_Chat_View extends StatefulWidget {
   var url;
   Map<String,String> post;
   Color Self_bgcolor;
   Color Other_bgcolor;
   Color self_fontcolor;
   Color other_fontcolor;

   String Send_message_url;
   int last_id = 0;
   int refresh_time = 0;

   $Ev_Chat_View({this.url,this.post,this.Self_bgcolor,this.Other_bgcolor,this.self_fontcolor,this.other_fontcolor,this.Send_message_url,this.refresh_time = 0,Key key}): super(key: key)
   {}

   @override
  $Ev_Chat_ViewtState createState() => $Ev_Chat_ViewtState();
}

class $Ev_Chat_ViewtState extends State<$Ev_Chat_View> implements Ev_FireBase_CallBack {

  Ev_API api = Ev_API();
  Send_Query send_query;
  var loaded = false;
  var error = false;
  Ev_Timer timer;
  List data;

  ScrollController _controller;

  void initState() {
    _controller = ScrollController();
    _controller.addListener(_scrollListener);
    send_query = Send_Query(chat_view: widget);
    timer = Ev_Timer();
    Ev_Firebase.initState_CallBack(this);
  }

  _scrollListener() {
    if (_controller.offset >= _controller.position.maxScrollExtent &&
        !_controller.position.outOfRange) {
    }
    if (_controller.offset <= _controller.position.minScrollExtent &&
        !_controller.position.outOfRange) {
    }
  }

  @override
  void dispose() {
    api.Stop();
    send_query.dispose();
    timer.stop();
    Ev_Firebase.dispose(this);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if(error)
    {  return _Error_Reload(context); }


    if(!loaded) {
      print(widget.post);
      Map<String,String> post = widget.post;
      post["last_id"] = "0";

      api.Post( widget.url, post: post, alert_error: true,
          onSuccess: (Ev_API_Result result) {
            widget.last_id = int.parse(result.sdata["last_id"]);
            setState(() {
              data = result.data;
              loaded = true;
              error = false;
            });
          },
          onConnectError: () {
            print("On Connect Error");
            setState(() {
              error = true;
              loaded = false;
            });
          },
          onReturnError:()
          {  print("On Return Error !");
            setState((){
            error = true;
            loaded = false;
          });
          }
      );
      return Center(
          child: SizedBox(
              child:
              new CircularProgressIndicator(
                  valueColor: new AlwaysStoppedAnimation(Colors.blue),
                  strokeWidth: 5.0),
              height: 40.0,
              width: 40.0));
    }
   else
    {
       if(widget.refresh_time>0)
         {  time_refresh(); }
       return _listview();


    }
  }

  void time_refresh()
  {  print("Chat Refresh:");
     Load_Message();
     timer.start(widget.refresh_time,widget.refresh_time, null, time_refresh);
  }


  Widget _Error_Reload(BuildContext context)
  {  return Center(child: Column(
    mainAxisAlignment: MainAxisAlignment.center,
    children: <Widget>[
      Icon(Icons.error_outline,color: Colors.amber,size: 110),
      RaisedButton(child: Icon(Icons.refresh,size: 25,),onPressed: (){
        setState(() {
          error = false;
          loaded = false;
        });
      },)
    ],),);
  }


  Widget _listview()
  { print("List Build item " + data.length.toString());
    return GestureDetector(
      child: ListView.builder(itemCount: data.length,
        padding: EdgeInsets.all(10.0),
        controller: _controller,
        reverse: true,
        itemBuilder: (context, index) {
        return Container(width: double.infinity,
            alignment: (data[index]["type"] == "1") ? Alignment.topRight : Alignment.topLeft,
            child: message_item(data[index])
        );
     }),
    onTap: (){ print("Tap "); FocusScope.of(context).requestFocus(FocusNode());

    Load_Message();
    },
    onTapDown: (TapDownDetails details)  { print("Tap Down"); FocusScope.of(context).requestFocus(FocusNode());},
   );
  }

  Widget message_item(sdata)
  {   if(sdata["type"] == "1" || sdata["member_icon"]=="")
        return message_item_body(sdata);
      else
        return Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
           Padding(
             padding: const EdgeInsets.only(top:5.0),
             child: ClipRRect(
                 borderRadius: BorderRadius.circular(15),
                 child: Image.network(sdata["member_icon"],width: 30,height: 30,fit: BoxFit.cover,),),
           ),
            message_item_body(sdata)
        ],);


  }

  Widget message_item_body(sdata) {

    Color fontcolor = (sdata["type"] == "1") ? widget.self_fontcolor : widget.other_fontcolor;
    Color bg_color = (sdata["type"] == "1") ? widget.Self_bgcolor : widget.Other_bgcolor;

    if (sdata["send_out"] == "1" ) {
       return sdata["widget"];
    }

    if (sdata["message_type"] == "0") {
      return Card(
        child: ClipRRect(
            borderRadius: BorderRadius.circular(5.0),
            child: Stack(
              children: <Widget>[
                Container(
                  padding: EdgeInsets.symmetric(horizontal: 10,vertical: 5),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[

                      (sdata["type"]) == "0" && sdata["member_name"]!="" ? Text(
                          sdata["member_name"],
                          style: TextStyle(color: Color(
                              int.parse(sdata["member_color"], radix: 16)))): Container(width:0),

                      Text(sdata["message"].toString(),
                        style: TextStyle(color: fontcolor),),


                      SizedBox(height: 13,),
                    ],
                  ),
                  color: bg_color,
                  constraints: BoxConstraints(
                    maxWidth: 300,
                    minWidth: 100,
                  ),
                ),

                Positioned(
                  right: 5,
                  bottom: 3,
                  child: Text(sdata["time"], style: TextStyle(fontSize: 10,
                      color: fontcolor.withOpacity(0.7)),
                    textAlign: TextAlign.right,),
                ),
              ],
            )),
      );
    }

    if (sdata["message_type"] == "1") {
      return Card(
        child: ClipRRect(
            borderRadius: BorderRadius.circular(5.0),
            child: Container(
              padding: EdgeInsets.all(4.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  (sdata["type"]) == "0" && sdata["member_name"]!="" ? Text(
                      sdata["member_name"],
                      style: TextStyle(color: Color(
                          int.parse(sdata["member_color"], radix: 16)))): Container(width:0),

                  Stack(children: <Widget>[
                    SizedBox(width: 200, height: 200,
                        child: GestureDetector(
                          child: Ev_Image_Widget(image: sdata["file"],
                            boxfit: BoxFit.cover,
                            round: 5,),
                          onTap: () {
                            Navigator.push(context, MaterialPageRoute(builder: (
                                context) => Ev_PhotoView(image: sdata["file"])));
                          },
                        )),
                    Positioned(right:5,bottom:0, child:
                    Padding(
                      padding: const EdgeInsets.all(5.0),
                      child: Text(sdata["time"],
                          style: TextStyle(fontSize: 10, color: Colors.white)),
                    ),)
                  ],),
                ],
              ),
              color: sdata["type"] == "1" ? widget.Self_bgcolor : widget.Other_bgcolor,
            )),
      );
    }

    if (sdata["message_type"] == "2") {
      return GestureDetector(
        onTap: (){ Ev_Func.Open_URL(sdata["file"]); },
        child: Stack(
          children: <Widget>[
            Card(
              child: ClipRRect(
                  borderRadius: BorderRadius.circular(5.0),
                  child: Container(
                      color: bg_color,
                      constraints: BoxConstraints(
                        maxWidth: 220,
                        minWidth: 100,),

                      padding: EdgeInsets.all(6.0),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                        (sdata["type"]) == "0" && sdata["member_name"]!="" ? Text(
                            sdata["member_name"],
                            style: TextStyle(color: Color(
                                int.parse(sdata["member_color"], radix: 16)))): Container(width:0),

                        SizedBox(height: 2,),
                        Row(children: <Widget>[
                          ClipRRect(borderRadius: BorderRadius.circular(15.0),
                              child: Container(width: 30, height: 30,
                                color: Colors.blue,
                                child: Center(child: Icon(
                                    Icons.insert_drive_file, size: 20,
                                    color: Colors.white)),)
                          ),

                          SizedBox(width: 8,),
                          Expanded(
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                              Text(sdata["filename"].toString(), overflow: TextOverflow.ellipsis,
                                style: TextStyle(color: fontcolor),),
                              Text(sdata["filesize"], style: TextStyle(fontSize: 10,
                                  color: fontcolor.withOpacity(0.7)),),
                            ],),
                          ),
                        ],),
                      ],))),
            ),
            Positioned(
              right: 15,
              bottom: 8,
              child: Text(sdata["time"], style: TextStyle(fontSize: 10,
                  color: fontcolor.withOpacity(0.7)),
                textAlign: TextAlign.right,),
            ),
          ],
        ),
         );
      }
  }


  void Send_Message(String message)
  {   if(message.replaceAll(" ","") == "" )
         { return; }
      Map<String,dynamic> sdata = Map();
      sdata["message"] = message;
      sdata["type"] = "1";
      sdata["send_out"] = "1";
      sdata["message_type"] = "0";
      sdata["time"] = DateTime.now().toString().substring(11,16);
      sdata["widget"] =  $EV_Send_Message(key:GlobalKey(),url: widget.Send_message_url,
      fontcolor: widget.self_fontcolor,
      bgcolor: widget.Self_bgcolor,
      post: widget.post,
      sdata: sdata,
      send_query: send_query,
      onSuccess: Send_Out_Received);

      setState(() {
        data.insert(0,sdata);
      });

  }

  void Send_Photo(File file)
  {    Map<String,dynamic> sdata = Map();
       sdata["message"] = "";
       sdata["type"] = "1";
       sdata["send_out"] = "1";
       sdata["photo"] = file;
       sdata["message_type"] = "1";
       sdata["time"] = DateTime.now().toString().substring(11,16);

       sdata["widget"] =  $EV_Send_Photo(key:GlobalKey(),url: widget.Send_message_url,
        fontcolor: widget.self_fontcolor,
        bgcolor: widget.Self_bgcolor,
        post: widget.post,
        sdata: sdata,
        send_query: send_query,
        onSuccess: Send_Out_Received);

     setState(() {
        data.insert(0,sdata);
     });

  }

  void Send_File(File file)
  {    Map<String,dynamic> sdata = Map();
       sdata["message"] = "";
       sdata["type"] = "1";
       sdata["send_out"] = "1";
       sdata["file"] = file;
       sdata["message_type"] = "1";
       sdata["time"] = DateTime.now().toString().substring(11,16);

      sdata["widget"] =  $EV_Send_File(key:GlobalKey(),url: widget.Send_message_url,
      fontcolor: widget.self_fontcolor,
      bgcolor: widget.Self_bgcolor,
      post: widget.post,
      sdata: sdata,
      send_query: send_query,
      onSuccess: Send_Out_Received);

  setState(() {
    data.insert(0,sdata);
  });


  }

  void Send_Out_Received(Ev_API_Result result)
   { print("Rescive " + result.toString()) ;
     var id = int.parse(result.sdata["last_id"]);
     if(id>widget.last_id)
         { widget.last_id = id; }

     setState(() {
       result.data.forEach((dynamic value) {
         data.insert(0,value);
       }); });
   }

  void Load_Message()
  { if(send_query.action == true)
       { return; }

    var post = widget.post;
         post["last_id"] = widget.last_id.toString();

     api.Post(widget.url, post: post,
      onSuccess: (Ev_API_Result result) {
       widget.last_id =  int.parse(result.sdata["last_id"]);
       setState(() {
          result.data.forEach((dynamic value) {
            data.insert(0,value);

          });

        if(result.data.length >0)
           {  //_controller.animateTo(0.0, duration: Duration(milliseconds: 300), curve: Curves.easeOut);

              SystemSound.play(SystemSoundType.click);
           }

        });
      });
   }

  @override
  FireBase_OnMessage(message) {
     Load_Message();
  }




}
